#include "pch.h"
#include "CenteredGlyphDumpRenderer.h"

#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace CenteredGlyphDump;

using namespace D2D1;
using namespace Microsoft::WRL;
using namespace Platform;

// Initializes D2D resources used for text rendering.
CenteredGlyphDumpRenderer::CenteredGlyphDumpRenderer(const std::shared_ptr<DeviceResources>& deviceResources) :
    m_deviceResources(deviceResources),
    m_needsRedraw(true)
{
    ZeroMemory(&m_glyphRun, sizeof(DWRITE_GLYPH_RUN));
    CreateDeviceDependentResources();
}

void CenteredGlyphDumpRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );
}
void CenteredGlyphDumpRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_stateBlock.Reset();
}

void CenteredGlyphDumpRenderer::CreateWindowSizeDependentResources()
{
    if (m_fontFace != nullptr)
    {
        DefineNewGlyphRun();
        m_needsRedraw = true;
    }
}

// Updates the text to be displayed.
void CenteredGlyphDumpRenderer::Update(DX::StepTimer const& timer)
{
}

// Renders a frame to the screen.
bool CenteredGlyphDumpRenderer::Render()
{
    if (!m_needsRedraw)
        return false;

    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
    context->SaveDrawingState(m_stateBlock.Get());
    context->BeginDraw();
    context->Clear(ColorF(ColorF::White));
    context->SetTransform(m_deviceResources->GetOrientationTransform2D());

    if (m_fontFace != nullptr)
    {
        context->DrawGlyphRun(Point2F(), &m_glyphRun, m_blackBrush.Get());
    }

    // We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    context->RestoreDrawingState(m_stateBlock.Get());

    m_needsRedraw = false;
    return true;
}

void CenteredGlyphDumpRenderer::SetFontFileKey(ComPtr<IDWriteFontFileLoader> fontFileLoader, String^ key)
{
    // Create custom font file reference
    ComPtr<IDWriteFontFile> fontFile;

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateCustomFontFileReference(&key, sizeof(String^), fontFileLoader.Get(), &fontFile)
        );

    // Create font face from custom font file
    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFace(DWRITE_FONT_FACE_TYPE_TRUETYPE,
                                                              1, // file count
                                                              fontFile.GetAddressOf(),
                                                              0,
                                                              DWRITE_FONT_SIMULATIONS_NONE,
                                                              &m_fontFace)
        );

    DefineNewGlyphRun();
    m_needsRedraw = true;
}

void CenteredGlyphDumpRenderer::DefineNewGlyphRun()
{
    // Create arrays for glyph indices, advances, offsets
    UINT16 glyphCount = m_fontFace->GetGlyphCount();
    m_glyphIndices = std::vector<uint16>(glyphCount);
    m_glyphAdvances = std::vector<float>(glyphCount);
    m_glyphOffsets = std::vector<DWRITE_GLYPH_OFFSET>(glyphCount);

    // Initialize indices and advances arrays
    for (uint16 index = 0; index < glyphCount; index++)
    {
        m_glyphIndices[index] = index;
        m_glyphAdvances[index] = 0;
    }

    // Get the vertical and horizontal font metrics;
    DWRITE_FONT_METRICS fontMetrics;
    m_fontFace->GetMetrics(&fontMetrics);

    std::vector<DWRITE_GLYPH_METRICS> glyphMetrics(glyphCount);

    DX::ThrowIfFailed(
        m_fontFace->GetDesignGlyphMetrics(m_glyphIndices.data(), glyphCount, glyphMetrics.data(), false)
        );

    // Get the output bounds, box size, and em size
    Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();
    int rows = (glyphCount + 16) / 16;
    float boxHeight = outputBounds.Height / (rows + 1);
    float boxWidth = outputBounds.Width / 17;
    float emSize = 0.75f * min(boxHeight, boxWidth);

    // Calculate glyph offsets
    for (uint16 index = 0; index < glyphCount; index++)
    {
        int col = index % 16;
        int row = index / 16;

        // Top center of character position
        float x = (col + 1) * boxWidth;
        float y = (row + 0.5f) * boxHeight;

        // Adjust for character metrics
        x -= emSize * glyphMetrics[index].advanceWidth / fontMetrics.designUnitsPerEm / 2;
        y += emSize * fontMetrics.ascent / fontMetrics.designUnitsPerEm;

        m_glyphOffsets[index].advanceOffset = x;
        m_glyphOffsets[index].ascenderOffset = -y;
    }

    // Define fields of DWRITE_GLYPH_RUN structure
    m_glyphRun.fontFace = m_fontFace.Get();
    m_glyphRun.fontEmSize = emSize;
    m_glyphRun.glyphCount = glyphCount;
    m_glyphRun.glyphIndices = m_glyphIndices.data();
    m_glyphRun.glyphAdvances = m_glyphAdvances.data();
    m_glyphRun.glyphOffsets = m_glyphOffsets.data();
    m_glyphRun.isSideways = false;
    m_glyphRun.bidiLevel = 0;

    m_needsRedraw = true;
}