﻿#include "pch.h"
#include "DirectXPage.xaml.h"
#include "Common\DirectXHelper.h"				// For ThrowIfFailed

using namespace FancyTitle;

using namespace D2D1;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::ApplicationModel;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::System::Threading;
using namespace Windows::UI::Core;
using namespace Windows::UI::Input;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

DirectXPage::DirectXPage():
	m_windowVisible(true)
{
	InitializeComponent();

	// Register event handlers for page lifecycle.
	CoreWindow^ window = Window::Current->CoreWindow;

	window->VisibilityChanged +=
		ref new TypedEventHandler<CoreWindow^, VisibilityChangedEventArgs^>(this, &DirectXPage::OnVisibilityChanged);

	DisplayInformation^ currentDisplayInformation = DisplayInformation::GetForCurrentView();

	currentDisplayInformation->DpiChanged +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnDpiChanged);

	currentDisplayInformation->OrientationChanged +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnOrientationChanged);

	DisplayInformation::DisplayContentsInvalidated +=
		ref new TypedEventHandler<DisplayInformation^, Object^>(this, &DirectXPage::OnDisplayContentsInvalidated);

	swapChainPanel->CompositionScaleChanged += 
		ref new TypedEventHandler<SwapChainPanel^, Object^>(this, &DirectXPage::OnCompositionScaleChanged);

	swapChainPanel->SizeChanged +=
		ref new SizeChangedEventHandler(this, &DirectXPage::OnSwapChainPanelSizeChanged);

	// At this point we have access to the device. 
	// We can create the device-dependent resources.
	m_deviceResources = std::make_shared<DeviceResources>();
	m_deviceResources->SetWindow(Window::Current->CoreWindow, swapChainPanel);

    // Create font face
    ComPtr<IDWriteFontFile> fontFile;
    String^ filepath = Package::Current->InstalledLocation->Path + "\\Fonts\\Pesca.ttf";

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFileReference(filepath->Data(), nullptr, &fontFile)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFace(
                DWRITE_FONT_FACE_TYPE_TRUETYPE,
                1, // file count
                fontFile.GetAddressOf(),
                0,
                DWRITE_FONT_SIMULATIONS_NONE,
                &m_fontFace)
        );

    CreateDeviceDependentResources();
}

// Saves the current state of the app for suspend and terminate events.
void DirectXPage::SaveInternalState(IPropertySet^ state)
{
	m_deviceResources->Trim();

	// Put code to save app state here.
}

// Loads the current state of the app for resume events.
void DirectXPage::LoadInternalState(IPropertySet^ state)
{
	// Put code to load app state here.
}


// Window event handlers.
void DirectXPage::OnVisibilityChanged(CoreWindow^ sender, VisibilityChangedEventArgs^ args)
{
	m_windowVisible = args->Visible;
}

// Display properties event handlers.
void DirectXPage::OnDpiChanged(DisplayInformation^ sender, Object^ args)
{
	m_deviceResources->SetDpi(sender->LogicalDpi);
    RedrawSwapChainPanel();
}

void DirectXPage::OnOrientationChanged(DisplayInformation^ sender, Object^ args)
{
	m_deviceResources->UpdateForWindowSizeChange();
    RedrawSwapChainPanel();
}

void DirectXPage::OnDisplayContentsInvalidated(DisplayInformation^ sender, Object^ args)
{
	m_deviceResources->ValidateDevice();
    RedrawSwapChainPanel();
}

void DirectXPage::OnCompositionScaleChanged(SwapChainPanel^ sender, Object^ args)
{
	m_deviceResources->CreateWindowSizeDependentResources();
    RedrawSwapChainPanel();
}

void DirectXPage::OnSwapChainPanelSizeChanged(Object^ sender, SizeChangedEventArgs^ e)
{
    m_deviceResources->UpdateForWindowSizeChange();
    RedrawSwapChainPanel();
}

void DirectXPage::OnDeviceLost()
{
    m_blackBrush.Reset();
}

void DirectXPage::OnDeviceRecreated()
{
    CreateDeviceDependentResources();
    RedrawSwapChainPanel();
}

void DirectXPage::CreateDeviceDependentResources()
{
    // Create brush
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
        );
}

void DirectXPage::RedrawSwapChainPanel()
{
    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
    context->BeginDraw();
    context->Clear(ColorF(ColorF::AliceBlue));
    context->SetTransform(m_deviceResources->GetOrientationTransform2D());

    std::array<uint16, 22> indices = 
    { 
        // "The Pescadero Difference"
        0x00DA, 0x0048, 0x0003,
        0x00CE, 0x0048, 0x0056, 0x0046, 0x0044, 0x0047, 0x0048, 0x0055, 0x0052, 0x0003,
        0x00C3, 0x004C, 0x00A6, 0x0048, 0x0055, 0x0048, 0x0051, 0x0046, 0x0048 
    };

    DWRITE_GLYPH_RUN glyphRun;
    glyphRun.fontFace = m_fontFace.Get();
    glyphRun.fontEmSize = 72.0f;
    glyphRun.glyphCount = indices.size();
    glyphRun.glyphIndices = indices.data();
    glyphRun.glyphAdvances = nullptr;
    glyphRun.glyphOffsets = nullptr;
    glyphRun.isSideways = false;
    glyphRun.bidiLevel = 0;

    context->DrawGlyphRun(Point2F(12, 52), &glyphRun, m_blackBrush.Get());

    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    // After rendering, present
    m_deviceResources->Present();
}
