﻿#include "pch.h"
#include "GlyphDumpRenderer.h"
#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace GlyphDump;

using namespace D2D1;
using namespace Microsoft::WRL;
using namespace Platform;

GlyphDumpRenderer::GlyphDumpRenderer(const std::shared_ptr<DeviceResources>& deviceResources) : 
	m_deviceResources(deviceResources),
    m_needsRedraw(true)
{
	CreateDeviceDependentResources();
}

void GlyphDumpRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );
}
void GlyphDumpRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_stateBlock.Reset();
}

void GlyphDumpRenderer::CreateWindowSizeDependentResources()
{
    m_needsRedraw = true;
}

void GlyphDumpRenderer::Update(DX::StepTimer const& timer)
{
}

// Renders a frame to the screen.
bool GlyphDumpRenderer::Render()
{
    if (!m_needsRedraw)
        return false;

	ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
	context->SaveDrawingState(m_stateBlock.Get());
	context->BeginDraw();
    context->Clear(ColorF(ColorF::White));
	context->SetTransform(m_deviceResources->GetOrientationTransform2D());

    if (m_fontFace != nullptr)
    {
        Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();
        uint16 glyphCount = m_fontFace->GetGlyphCount();
        int rows = (glyphCount + 16) / 16;
        float boxHeight = outputBounds.Height / (rows + 1);
        float boxWidth = outputBounds.Width / 17;

        // Define entire structure except glyphIndices
        DWRITE_GLYPH_RUN glyphRun;
        glyphRun.fontFace = m_fontFace.Get();
        glyphRun.fontEmSize = 0.75f * min(boxHeight, boxWidth);
        glyphRun.glyphCount = 1;
        glyphRun.glyphAdvances = nullptr;
        glyphRun.glyphOffsets = nullptr;
        glyphRun.isSideways = false;
        glyphRun.bidiLevel = 0;

        for (uint16 index = 0; index < glyphCount; index++)
        {
            glyphRun.glyphIndices = &index;

            context->DrawGlyphRun(Point2F(((index) % 16 + 0.5f) * boxWidth, 
                                          ((index) / 16 + 1) * boxHeight), 
                                          &glyphRun, 
                                          m_blackBrush.Get());
        }
    }

	// We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
	// is lost. It will be handled during the next call to Present.
	HRESULT hr = context->EndDraw();
	if (hr != D2DERR_RECREATE_TARGET)
	{
		DX::ThrowIfFailed(hr);
	}

	context->RestoreDrawingState(m_stateBlock.Get());

    m_needsRedraw = false;
    return true;
}

void GlyphDumpRenderer::SetFontFilePath(std::wstring filepath)
{
    ComPtr<IDWriteFontFile> fontFile;

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFileReference(filepath.c_str(), nullptr, &fontFile)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFace(DWRITE_FONT_FACE_TYPE_TRUETYPE,
                                                              1, // file count
                                                              fontFile.GetAddressOf(),
                                                              0,
                                                              DWRITE_FONT_SIMULATIONS_NONE,
                                                              &m_fontFace)
        );

    m_needsRedraw = true;
}