﻿#include "pch.h"
#include "HelloGlyphRunRenderer.h"
#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace HelloGlyphRun;

using namespace D2D1;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::ApplicationModel;

HelloGlyphRunRenderer::HelloGlyphRunRenderer(const std::shared_ptr<DeviceResources>& deviceResources) : 
	m_deviceResources(deviceResources)
{
	CreateDeviceDependentResources();
}

void HelloGlyphRunRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
        );
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );
}
void HelloGlyphRunRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_stateBlock.Reset();
}

void HelloGlyphRunRenderer::CreateWindowSizeDependentResources()
{
    ComPtr<IDWriteFontFile> fontFile;

    String^ filepath = Package::Current->InstalledLocation->Path + "\\Fonts\\Pesca.ttf";

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFileReference(filepath->Data(), nullptr, &fontFile)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateFontFace(DWRITE_FONT_FACE_TYPE_TRUETYPE,
        1, // file count
        fontFile.GetAddressOf(),
        0,
        DWRITE_FONT_SIMULATIONS_NONE,
        &m_fontFace)
        );

    // Convert string to glyph indices
    std::wstring str = L"Hello, Glyph Run!";
    uint32 glyphCount = str.length();
    std::vector<uint32> str32(glyphCount);

    for (uint32 i = 0; i < glyphCount; i++)
        str32[i] = str[i];

    m_glyphIndices = std::vector<uint16>(glyphCount);
    m_fontFace->GetGlyphIndices(str32.data(), glyphCount, m_glyphIndices.data());

    // Allocate array for offsets (set during Update)
    m_glyphOffsets = std::vector<DWRITE_GLYPH_OFFSET>(glyphCount);

    // Get output bounds
    Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();

    // Define fields of DWRITE_GLYPH_RUN structure
    m_glyphRun.fontFace = m_fontFace.Get();
    m_glyphRun.fontEmSize = outputBounds.Width / 8; // empirical
    m_glyphRun.glyphCount = glyphCount;
    m_glyphRun.glyphIndices = m_glyphIndices.data();
    m_glyphRun.glyphAdvances = nullptr;
    m_glyphRun.glyphOffsets = m_glyphOffsets.data();
    m_glyphRun.isSideways = false;
    m_glyphRun.bidiLevel = 0;

    m_maxMove = outputBounds.Width / 20;
}

// Updates the text to be displayed.
void HelloGlyphRunRenderer::Update(DX::StepTimer const& timer)
{
    // Calculate t from 0 to 1 and back to 0 over 3 seconds
    float t = float(fmod(timer.GetTotalSeconds(), 3.0) / 3.0);
    t = t < 0.5f ? 2 * t : 2 - 2 * t;

    // Set offsets for ripple effect
    int count = m_glyphOffsets.size();

    for (int i = 0; i < count; i++)
    {
        m_glyphOffsets[i].advanceOffset = max(0, min(m_maxMove, (m_maxMove / 5) * ((count + 4) * t - i)));
        m_glyphOffsets[i].ascenderOffset = 0;
    }
}

// Renders a frame to the screen.
void HelloGlyphRunRenderer::Render()
{
	ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
	Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();

	context->SaveDrawingState(m_stateBlock.Get());
	context->BeginDraw();
    context->Clear(ColorF(ColorF::White));
	context->SetTransform(m_deviceResources->GetOrientationTransform2D());

    context->DrawGlyphRun(Point2F(0, outputBounds.Height / 2), &m_glyphRun, m_blackBrush.Get());

	// We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
	// is lost. It will be handled during the next call to Present.
	HRESULT hr = context->EndDraw();
	if (hr != D2DERR_RECREATE_TARGET)
	{
		DX::ThrowIfFailed(hr);
	}

	context->RestoreDrawingState(m_stateBlock.Get());
}

