﻿#include "pch.h"
#include "OutlinedCharactersRenderer.h"
#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace OutlinedCharacters;

using namespace D2D1;
using namespace Microsoft::WRL;

OutlinedCharactersRenderer::OutlinedCharactersRenderer(const std::shared_ptr<DeviceResources>& deviceResources) :
    m_deviceResources(deviceResources)
{
    CreateDeviceDependentResources();
}

void OutlinedCharactersRenderer::Update(DX::StepTimer const& timer)
{
    // Create animated dot style
    double seconds = timer.GetTotalSeconds();
    float dotOffset = float(fmod(2 * seconds, 2));

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateStrokeStyle(
                StrokeStyleProperties(D2D1_CAP_STYLE_ROUND,
                                      D2D1_CAP_STYLE_ROUND,
                                      D2D1_CAP_STYLE_ROUND,
                                      D2D1_LINE_JOIN_ROUND,
                                      10.0f, D2D1_DASH_STYLE_DOT,
                                      dotOffset),
                                      nullptr, 
                                      0, 
                                      &m_animatedStrokeStyle)
        );
}

// Renders a frame to the screen.
void OutlinedCharactersRenderer::Render()
{
    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
    Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();

    context->SaveDrawingState(m_stateBlock.Get());
    context->BeginDraw();
    context->Clear(D2D1::ColorF(D2D1::ColorF::AliceBlue));

    if (m_characterGeometry != nullptr)
    {
        D2D1_RECT_F geometryBounds;

        DX::ThrowIfFailed(
            m_characterGeometry->GetBounds(D2D1::IdentityMatrix(), &geometryBounds)
            );

        Matrix3x2F translation = Matrix3x2F::Translation(
            (outputBounds.Width - geometryBounds.right + geometryBounds.left) / 2,
            (outputBounds.Height / 3 - geometryBounds.bottom - geometryBounds.top) / 2);

        // Display the character geometry
        context->SetTransform(translation * m_deviceResources->GetOrientationTransform2D());
        context->FillGeometry(m_characterGeometry.Get(), m_blueBrush.Get());
        context->DrawGeometry(m_characterGeometry.Get(), m_redBrush.Get(), 10);

        // Display the character geometry with moving dots
        translation._32 += outputBounds.Height / 3;
        context->SetTransform(translation * m_deviceResources->GetOrientationTransform2D());
        context->DrawGeometry(m_characterGeometry.Get(), m_blackBrush.Get(), 10, m_animatedStrokeStyle.Get());

        // Display the widened and outlined geometry
        translation._32 += outputBounds.Height / 3;
        context->SetTransform(translation * m_deviceResources->GetOrientationTransform2D());
        context->FillGeometry(m_outlinedGeometry.Get(), m_blueBrush.Get());
        context->DrawGeometry(m_outlinedGeometry.Get(), m_redBrush.Get(), 5);
    }

    // We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    context->RestoreDrawingState(m_stateBlock.Get());
}

void OutlinedCharactersRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Blue), &m_blueBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Red), &m_redBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );
}
void OutlinedCharactersRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_blueBrush.Reset();
    m_redBrush.Reset();
    m_stateBlock.Reset();
}

void OutlinedCharactersRenderer::CreateWindowSizeDependentResources()
{
    if (m_fontFace != nullptr)
        CreateGlyphOutlines();
}

void OutlinedCharactersRenderer::SetFontFace(ComPtr<IDWriteFontFace> fontFace)
{
    m_fontFace = fontFace;

    if (m_deviceResources->GetOutputBounds().Width > 0)
        CreateGlyphOutlines();
}

void OutlinedCharactersRenderer::CreateGlyphOutlines()
{
    // Convert text to glyph indices
    std::wstring str = L"Outline";
    uint32 glyphCount = str.length();
    std::vector<uint32> codePoints(glyphCount);

    for (uint32 i = 0; i < glyphCount; i++)
        codePoints[i] = str[i];

    std::vector<uint16> glyphIndices(glyphCount);

    DX::ThrowIfFailed(
        m_fontFace->GetGlyphIndices(codePoints.data(), glyphCount, glyphIndices.data())
        );

    // Calculate em size to fit output bounds.
    Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();
    float emSize = min(outputBounds.Width / 4.0f, outputBounds.Height / 3.0f);

    // Get ID2D1 factory
    ID2D1Factory2 * factory = m_deviceResources->GetD2DFactory();

    // Create path geometry based on character outlines
    DX::ThrowIfFailed(
        factory->CreatePathGeometry(&m_characterGeometry)
        );

    ComPtr<ID2D1GeometrySink> geometrySink;

    DX::ThrowIfFailed(
        m_characterGeometry->Open(&geometrySink)
        );

    DX::ThrowIfFailed(
        m_fontFace->GetGlyphRunOutline(emSize, glyphIndices.data(), nullptr, nullptr, glyphCount, false, false, geometrySink.Get())
        );

    DX::ThrowIfFailed(
        geometrySink->Close()
        );

    // Create widened geometry
    ComPtr<ID2D1PathGeometry> widenedGeometry;

    DX::ThrowIfFailed(
        factory->CreatePathGeometry(&widenedGeometry)
        );

    DX::ThrowIfFailed(
        widenedGeometry->Open(&geometrySink)
        );

    ComPtr<ID2D1StrokeStyle> strokeStyle;

    DX::ThrowIfFailed(
        factory->CreateStrokeStyle(StrokeStyleProperties(), nullptr, 0, &strokeStyle)
        );

    DX::ThrowIfFailed(
        m_characterGeometry->Widen(emSize / 25, strokeStyle.Get(), D2D1::IdentityMatrix(), geometrySink.Get())
        );

    DX::ThrowIfFailed(
        geometrySink->Close()
        );

    // Create outlined geometry based on the widened geometry
    DX::ThrowIfFailed(
        factory->CreatePathGeometry(&m_outlinedGeometry)
        );

    DX::ThrowIfFailed(
        m_outlinedGeometry->Open(&geometrySink)
        );

    DX::ThrowIfFailed(
        widenedGeometry->Outline(D2D1::IdentityMatrix(), geometrySink.Get())
        );

    DX::ThrowIfFailed(
        geometrySink->Close()
        );
}