﻿#include "pch.h"
#include "ResourceFontLayoutRenderer.h"

#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace ResourceFontLayout;

using namespace D2D1;
using namespace Microsoft::WRL;

// Initializes D2D resources used for text rendering.
ResourceFontLayoutRenderer::ResourceFontLayoutRenderer(const std::shared_ptr<DeviceResources>& deviceResources) : 
    m_needsRedraw(true),
	m_deviceResources(deviceResources)
{
	CreateDeviceDependentResources();
}

void ResourceFontLayoutRenderer::CreateDeviceDependentResources()
{
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );
}
void ResourceFontLayoutRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
    m_stateBlock.Reset();
}

void ResourceFontLayoutRenderer::CreateWindowSizeDependentResources()
{
    m_needsRedraw = true;
}

void ResourceFontLayoutRenderer::Update(DX::StepTimer const& timer)
{
}

// Renders a frame to the screen.
bool ResourceFontLayoutRenderer::Render()
{
    if (!m_needsRedraw)
        return false;

	ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
	Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();

	context->SaveDrawingState(m_stateBlock.Get());
	context->BeginDraw();
    context->Clear(ColorF(ColorF::White));

    if (m_textLayout != nullptr)
    {
        // Position in the center
        Matrix3x2F screenTranslation = Matrix3x2F::Translation(
                        (outputBounds.Width - m_textMetrics.layoutWidth) / 2,
                        (outputBounds.Height - m_textMetrics.height) / 2);

        context->SetTransform(screenTranslation * m_deviceResources->GetOrientationTransform2D());

        context->DrawTextLayout(Point2F(0.f, 0.f),
                                m_textLayout.Get(),
                                m_blackBrush.Get());
    }
	// We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
	// is lost. It will be handled during the next call to Present.
	HRESULT hr = context->EndDraw();
	if (hr != D2DERR_RECREATE_TARGET)
	{
		DX::ThrowIfFailed(hr);
	}

	context->RestoreDrawingState(m_stateBlock.Get());

    m_needsRedraw = false;
    return true;
}

void ResourceFontLayoutRenderer::SetFontFamilyName(ComPtr<IDWriteFontCollection> fontCollection, std::wstring familyName)
{
    ComPtr<IDWriteFactory> writeFactory = m_deviceResources->GetDWriteFactory();

    DX::ThrowIfFailed(
        writeFactory->CreateTextFormat(familyName.c_str(),
                                       fontCollection.Get(),
                                       DWRITE_FONT_WEIGHT_NORMAL,
                                       DWRITE_FONT_STYLE_NORMAL,
                                       DWRITE_FONT_STRETCH_NORMAL,
                                       32.0f,
                                       L"en-US",
                                       &m_textFormat)
        );

    std::wstring text = L"This is a font in the " +
                        familyName +
                        L" family. It is capable of italic and bold " +
                        L"using either an actual font or synthesized " +
                        L"during rendering.";

    DX::ThrowIfFailed(
        writeFactory->CreateTextLayout(text.c_str(), text.size(), m_textFormat.Get(), 400, 1000, &m_textLayout)
        );

    int index = text.find(L"italic", 0);
    DWRITE_TEXT_RANGE textRange = { index, 6 };
    m_textLayout->SetFontStyle(DWRITE_FONT_STYLE_ITALIC, textRange);

    index = text.find(L"bold", 0);
    textRange = { index, 4 };
    m_textLayout->SetFontWeight(DWRITE_FONT_WEIGHT_BOLD, textRange);

    DX::ThrowIfFailed(
        m_textLayout->GetMetrics(&m_textMetrics)
        );

    m_needsRedraw = true;
}