#include "pch.h"
#include "PrivateFontFileLoader.h"
#include "PrivateFontFileStream.h"

using namespace Concurrency;
using namespace Microsoft::WRL;
using namespace Platform;
using namespace Windows::Foundation::Collections;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;

ComPtr<PrivateFontFileLoader> PrivateFontFileLoader::m_fontLoader = nullptr;

// Private constructor
PrivateFontFileLoader::PrivateFontFileLoader() : m_refCount(0)
{
}

// IUnknown methods
ULONG STDMETHODCALLTYPE PrivateFontFileLoader::AddRef()
{
    m_refCount++;
    return m_refCount;
}

ULONG STDMETHODCALLTYPE PrivateFontFileLoader::Release()
{
    m_refCount--;
    LONG newCount = m_refCount;

    if (m_refCount == 0)
        delete this;

    return newCount;
}

HRESULT STDMETHODCALLTYPE PrivateFontFileLoader::QueryInterface(_In_ REFIID riid, _Outptr_ void** ppOutput)
{
    *ppOutput = nullptr;
    HRESULT hr = S_OK;

    if (riid == __uuidof(IDWriteFontFileLoader))
    {
        *ppOutput = static_cast<IDWriteFontFileLoader*>(this);
        AddRef();
    }
    else if (riid == __uuidof(IUnknown))
    {
        *ppOutput = this;
        AddRef();
    }
    else
    {
        hr = E_NOINTERFACE;
    }
    return hr;
}

// Static public creation method to implement singleton model
PrivateFontFileLoader * PrivateFontFileLoader::Create()
{
    if (m_fontLoader == nullptr)
        m_fontLoader = new PrivateFontFileLoader();

    return m_fontLoader.Get();
}

// Public asynchronous method
task<void> PrivateFontFileLoader::LoadFontsAsync(StorageFolder^ storageFolder, String^ subDirectory)
{
    task<StorageFolder^> getFolderTask = create_task(storageFolder->GetFolderAsync(subDirectory));
    return getFolderTask.then([this](StorageFolder^ fontFolder)
    {
        return fontFolder->GetFilesAsync();

    }).then([this](IVectorView<StorageFile^>^ fontFiles)
    {
        std::vector<task<FileNameAndBuffer>> readFileTasks;

        for (StorageFile^ fontFile : fontFiles)
        {
            readFileTasks.push_back(task<FileNameAndBuffer>(ReadBufferAsync(fontFile)));
        }

        return when_all(readFileTasks.begin(), readFileTasks.end());

    }).then([this](std::vector<FileNameAndBuffer> fileNameAndBuffers)
    {
        for (FileNameAndBuffer fileNameAndBuffer : fileNameAndBuffers)
        {
            Array<byte>^ data = ref new Array<byte>(fileNameAndBuffer.buffer->Length);
            DataReader^ dataReader = DataReader::FromBuffer(fileNameAndBuffer.buffer);
            dataReader->ReadBytes(data);
            ComPtr<PrivateFontFileStream> fontFileStream = new PrivateFontFileStream(data);

            // Save in the std::map and std::vector
            m_fileStreams.insert(std::make_pair(fileNameAndBuffer.fileName, fontFileStream));
            m_fileNames.push_back(fileNameAndBuffer.fileName);
        }
    });
}

// This method exists primarily to retain the filenames through the when_all
task<FileNameAndBuffer> PrivateFontFileLoader::ReadBufferAsync(StorageFile^ fontFile)
{
    task<IBuffer^> readFileTask = create_task(FileIO::ReadBufferAsync(fontFile));
    return readFileTask.then([fontFile](IBuffer^ buffer)
    {
        FileNameAndBuffer nameAndBuffer;
        nameAndBuffer.fileName = fontFile->Name;
        nameAndBuffer.buffer = buffer;
        return nameAndBuffer;
    });
}

// These are the file keys!
std::vector<String^> PrivateFontFileLoader::GetFileNames()
{
    return m_fileNames;
}

// IDWriteFontFileLoader method
HRESULT STDMETHODCALLTYPE PrivateFontFileLoader::CreateStreamFromKey(_In_ const void * fontFileReferenceKey,
                                                                     UINT32 fontFileReferenceKeySize,
                                                                     _Out_ IDWriteFontFileStream ** fontFileStream)
{
    if (fontFileReferenceKeySize != sizeof(String^) )
        return E_INVALIDARG;

    String^ key = *(static_cast<String^ const *>(fontFileReferenceKey));

    // Create new pointer detached from the internal reference-counted pointer
    *fontFileStream = ComPtr<IDWriteFontFileStream>(m_fileStreams.at(key).Get()).Detach();
    return S_OK;
}

