﻿#include "pch.h"
#include "SystemFontGlyphsRenderer.h"
#include "Common/DirectXHelper.h"	// For ThrowIfFailed

using namespace SystemFontGlyphs;

using namespace Microsoft::WRL;

SystemFontGlyphsRenderer::SystemFontGlyphsRenderer(const std::shared_ptr<DeviceResources>& deviceResources) : 
	m_deviceResources(deviceResources)
{
	CreateDeviceDependentResources();
}

void SystemFontGlyphsRenderer::Update(DX::StepTimer const& timer)
{
    // Set glyph offsets to random numbers between -3 and +3.
    if (m_glyphOffsets.size() > 0)
    {
        for (DWRITE_GLYPH_OFFSET& glyphOffset : m_glyphOffsets)
        {
            glyphOffset.advanceOffset = rand() % 7 - 3.0f;
            glyphOffset.ascenderOffset = rand() % 7 - 3.0f;
        }
    }
}

// Renders a frame to the screen.
void SystemFontGlyphsRenderer::Render()
{
	ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
	Windows::Foundation::Size outputBounds = m_deviceResources->GetOutputBounds();

	context->SaveDrawingState(m_stateBlock.Get());
	context->BeginDraw();
    context->Clear(D2D1::ColorF(D2D1::ColorF::AliceBlue));
	context->SetTransform(m_deviceResources->GetOrientationTransform2D());

    if (m_glyphRun.glyphCount > 0)
    {
        // Center the glyph run
        Windows::Foundation::Size size = m_deviceResources->GetOutputBounds();
        float x = (size.Width - m_glyphRunWidth) / 2;
        float y = (size.Height + m_glyphRunAscent - m_glyphRunDescent) / 2;

        // Draw it!
        context->DrawGlyphRun(D2D1::Point2F(x, y), &m_glyphRun, m_blackBrush.Get());
    }

	// We ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
	// is lost. It will be handled during the next call to Present.
	HRESULT hr = context->EndDraw();
	if (hr != D2DERR_RECREATE_TARGET)
	{
		DX::ThrowIfFailed(hr);
	}

	context->RestoreDrawingState(m_stateBlock.Get());
}

void SystemFontGlyphsRenderer::CreateDeviceDependentResources()
{
	DX::ThrowIfFailed(
		m_deviceResources->GetD2DDeviceContext()->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Black), &m_blackBrush)
		);

	DX::ThrowIfFailed(
		m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
		);
}
void SystemFontGlyphsRenderer::ReleaseDeviceDependentResources()
{
	m_blackBrush.Reset();
	m_stateBlock.Reset();
}

void SystemFontGlyphsRenderer::CreateWindowSizeDependentResources()
{
    if (m_fontFace != nullptr)
        CreateGlyphRun();
}

void SystemFontGlyphsRenderer::SetFontFace(ComPtr<IDWriteFontFace> fontFace)
{
    m_fontFace = fontFace;

    if (m_deviceResources->GetOutputBounds().Width > 0)
        CreateGlyphRun();
}

void SystemFontGlyphsRenderer::CreateGlyphRun()
{
    // Convert text to glyph indices
    std::wstring str = L"Annoying vibrating text effect";
    uint32 glyphCount = str.length();
    std::vector<uint32> codePoints(glyphCount);

    for (uint32 i = 0; i < glyphCount; i++)
        codePoints[i] = str[i];

    m_glyphIndices = std::vector<uint16>(glyphCount);
    m_glyphOffsets = std::vector<DWRITE_GLYPH_OFFSET>(glyphCount);

    DX::ThrowIfFailed(
        m_fontFace->GetGlyphIndices(codePoints.data(), glyphCount, m_glyphIndices.data())
        );

    // Notice em size based on width of output surface!
    float emSize = m_deviceResources->GetOutputBounds().Width / 15;

    m_glyphRun.fontFace = m_fontFace.Get();
    m_glyphRun.fontEmSize = emSize;
    m_glyphRun.glyphCount = glyphCount;
    m_glyphRun.glyphIndices = m_glyphIndices.data();
    m_glyphRun.glyphAdvances = nullptr;
    m_glyphRun.glyphOffsets = m_glyphOffsets.data();
    m_glyphRun.isSideways = false;
    m_glyphRun.bidiLevel = 0;

    // Get glyph metrics
    DWRITE_FONT_METRICS fontMetrics;
    m_fontFace->GetMetrics(&fontMetrics);
    m_glyphRunAscent = emSize * fontMetrics.ascent / fontMetrics.designUnitsPerEm;
    m_glyphRunDescent = emSize * fontMetrics.descent / fontMetrics.designUnitsPerEm;

    std::vector<DWRITE_GLYPH_METRICS> glyphRunMetrics(glyphCount);

    DX::ThrowIfFailed(
        m_fontFace->GetDesignGlyphMetrics(m_glyphIndices.data(), glyphCount, glyphRunMetrics.data())
        );

    // Determine total width of glyph run
    m_glyphRunWidth = 0;
    for (DWRITE_GLYPH_METRICS glyphMetrics : glyphRunMetrics)
        m_glyphRunWidth += emSize * glyphMetrics.advanceWidth / fontMetrics.designUnitsPerEm;
}